import React, { useState } from 'react';

const PatientTemplate = () => {
  const [formData, setFormData] = useState({
    identifier: '',
    active: '',
    name: '',
    telecom: '',
    gender: '',
    birthDate: '',
    deceasedX: '',
    address: '',
    maritalStatus: '',
    multipleBirthX: '',
    photo: '',
    contact: {
      relationship: '',
      role: '',
      name: '',
      additionalName: '',
      telecom: '',
      address: '',
      additionalAddress: '',
      gender: '',
      organization: '',
      period: ''
    },
    communication: {
      language: '',
      preferred: ''
    },
    generalPractitioner: '',
    managingOrganization: '',
    link: {
      other: '',
      type: ''
    }
  });

  const handleInputChange = (path, value) => {
    const pathArray = path.split('.');
    setFormData(prev => {
      const newData = { ...prev };
      let current = newData;
      for (let i = 0; i < pathArray.length - 1; i++) {
        current = current[pathArray[i]];
      }
      current[pathArray[pathArray.length - 1]] = value;
      return newData;
    });
  };

  const renderField = (label, path, comment, type = 'text', options = null) => {
    const value = path.split('.').reduce((obj, key) => obj[key], formData);
    
    return (
      <div className="mb-4 border-l-2 border-gray-200 pl-4">
        <div className="flex items-start mb-1">
          <label className="text-sm font-medium text-gray-700 mr-2">
            {label}
          </label>
          <span className="text-xs text-gray-500 italic">
            {comment}
          </span>
        </div>
        {options ? (
          <select
            value={value}
            onChange={(e) => handleInputChange(path, e.target.value)}
            className="w-full p-2 border rounded-md"
          >
            <option value="">Select...</option>
            {options.map(opt => (
              <option key={opt} value={opt}>{opt}</option>
            ))}
          </select>
        ) : (
          <input
            type={type}
            value={value}
            onChange={(e) => handleInputChange(path, e.target.value)}
            placeholder={type === 'boolean' ? '[boolean]' : type === 'date' ? '[date]' : ''}
            className="w-full p-2 border rounded-md"
          />
        )}
      </div>
    );
  };

  return (
    <div className="bg-white p-6 rounded-lg shadow">
      <h2 className="text-2xl font-bold mb-6">Patient Resource Template</h2>
      <div className="space-y-6">
        {/* Resource and DomainResource comment */}
        <div className="text-sm text-gray-500 mb-4">
          <p>from Resource: id, meta, implicitRules, and language</p>
          <p>from DomainResource: text, contained, extension, and modifierExtension</p>
        </div>

        {/* Basic Information */}
        {renderField('Identifier', 'identifier', '0..* Identifier An identifier for this patient')}
        {renderField('Active', 'active', '0..1 Whether this patient\'s record is in active use', 'boolean')}
        {renderField('Name', 'name', '0..* HumanName A name associated with the patient')}
        {renderField('Telecom', 'telecom', '0..* ContactPoint A contact detail for the individual')}
        {renderField('Gender', 'gender', '0..1 male | female | other | unknown', 'select', ['male', 'female', 'other', 'unknown'])}
        {renderField('Birth Date', 'birthDate', '0..1 The date of birth for the individual', 'date')}
        {renderField('Deceased', 'deceasedX', '0..1 boolean|dateTime Indicates if/when the individual is deceased')}
        {renderField('Address', 'address', '0..* Address An address for the individual')}
        {renderField('Marital Status', 'maritalStatus', '0..1 CodeableConcept Marital (civil) status of a patient')}
        {renderField('Multiple Birth', 'multipleBirthX', '0..1 boolean|integer Whether patient is part of a multiple birth')}
        {renderField('Photo', 'photo', '0..* Attachment Image of the patient')}

        {/* Contact Information */}
        <div className="border-l-2 border-blue-200 pl-4 mb-4">
          <h3 className="text-lg font-semibold mb-2">Contact</h3>
          <div className="text-xs text-gray-500 mb-2">0..* A contact party (e.g. guardian, partner, friend) for the patient</div>
          {renderField('Relationship', 'contact.relationship', '0..* CodeableConcept The kind of personal relationship icon')}
          {renderField('Role', 'contact.role', '0..* CodeableConcept The kind of functional role')}
          {renderField('Name', 'contact.name', '0..1 HumanName A name associated with the contact person')}
          {renderField('Additional Name', 'contact.additionalName', '0..* HumanName Additional names for the contact person')}
          {renderField('Telecom', 'contact.telecom', '0..* ContactPoint A contact detail for the person')}
          {renderField('Address', 'contact.address', '0..1 Address Address for the contact person')}
          {renderField('Additional Address', 'contact.additionalAddress', '0..* Address Additional addresses for the contact person')}
          {renderField('Gender', 'contact.gender', '0..1 male | female | other | unknown', 'select', ['male', 'female', 'other', 'unknown'])}
          {renderField('Organization', 'contact.organization', '0..1 Reference(Organization) Organization that is associated with the contact')}
          {renderField('Period', 'contact.period', '0..1 Period The period during which this contact person or organization is valid')}
        </div>

        {/* Communication */}
        <div className="border-l-2 border-green-200 pl-4 mb-4">
          <h3 className="text-lg font-semibold mb-2">Communication</h3>
          <div className="text-xs text-gray-500 mb-2">0..* A language which may be used to communicate with the patient about his or her health</div>
          {renderField('Language', 'communication.language', '1..1 CodeableConcept The language which can be used to communicate')}
          {renderField('Preferred', 'communication.preferred', '0..1 Language preference indicator', 'boolean')}
        </div>

        {/* References */}
        {renderField('General Practitioner', 'generalPractitioner', '0..* Reference(Organization|Practitioner|PractitionerRole) Patient\'s nominated primary care provider')}
        {renderField('Managing Organization', 'managingOrganization', '0..1 Reference(Organization) Organization that is the custodian of the patient record')}

        {/* Link */}
        <div className="border-l-2 border-purple-200 pl-4 mb-4">
          <h3 className="text-lg font-semibold mb-2">Link</h3>
          <div className="text-xs text-gray-500 mb-2">0..* Link to a Patient or RelatedPerson resource that concerns the same actual individual</div>
          {renderField('Other', 'link.other', '1..1 Reference(Patient|RelatedPerson) The other patient or related person resource')}
          {renderField('Type', 'link.type', '1..1 replaced-by | replaces | refer | seealso', 'select', ['replaced-by', 'replaces', 'refer', 'seealso'])}
        </div>
      </div>
    </div>
  );
};

export default PatientTemplate; 