import React, { useState, useEffect } from 'react';
import PropTypes from 'prop-types';

const XMLResourceViewer = ({ resourceKey }) => {
  const [resource, setResource] = useState(null);
  const [schema, setSchema] = useState(null);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);

  useEffect(() => {
    const fetchResource = async () => {
      try {
        setLoading(true);
        const [resourceResponse, schemaResponse] = await Promise.all([
          fetch(`http://localhost:8080/api/resources/${resourceKey}`),
          fetch(`http://localhost:8080/api/resources/${resourceKey}/schema`)
        ]);

        if (!resourceResponse.ok || !schemaResponse.ok) {
          throw new Error('Failed to fetch resource data');
        }

        const [resourceData, schemaData] = await Promise.all([
          resourceResponse.json(),
          schemaResponse.json()
        ]);

        setResource(resourceData);
        setSchema(schemaData);
        setError(null);
      } catch (err) {
        setError(err.message);
      } finally {
        setLoading(false);
      }
    };

    if (resourceKey) {
      fetchResource();
    }
  }, [resourceKey]);

  const renderValue = (value, path = '') => {
    if (value === null || value === undefined) {
      return <input type="text" placeholder="Empty" className="w-full p-1 border rounded" />;
    }

    if (typeof value === 'object') {
      if (Array.isArray(value)) {
        return (
          <div className="ml-4">
            {value.map((item, index) => (
              <div key={`${path}_${index}`} className="mb-2">
                <div className="font-semibold text-gray-600">Item {index + 1}</div>
                {renderValue(item, `${path}_${index}`)}
              </div>
            ))}
          </div>
        );
      }

      return (
        <div className="ml-4">
          {Object.entries(value).map(([key, val]) => (
            <div key={`${path}_${key}`} className="mb-2">
              <div className="font-semibold text-gray-600">{key}</div>
              {renderValue(val, `${path}_${key}`)}
            </div>
          ))}
        </div>
      );
    }

    return (
      <input
        type="text"
        value={String(value)}
        onChange={(e) => {
          // TODO: Implement change handling
          console.log('Value changed:', e.target.value);
        }}
        className="w-full p-1 border rounded"
      />
    );
  };

  if (loading) {
    return <div className="p-4">Loading...</div>;
  }

  if (error) {
    return <div className="p-4 text-red-500">Error: {error}</div>;
  }

  if (!resource) {
    return <div className="p-4">No resource selected</div>;
  }

  return (
    <div className="p-4">
      <h2 className="text-2xl font-bold mb-4">
        {resource.resourceType} - {resource.metadata.filename}
      </h2>
      <div className="bg-white rounded-lg shadow p-6">
        {renderValue(resource.content)}
      </div>
    </div>
  );
};

XMLResourceViewer.propTypes = {
  resourceKey: PropTypes.string.isRequired,
};

export default XMLResourceViewer; 