from __future__ import annotations

import xml.etree.ElementTree as ET
from typing import Optional, Dict, Any
import logging
from datetime import datetime
import uuid

from fhir.resources.bundle import Bundle
from fhir.resources.messageheader import MessageHeader
from fhir.resources.patient import Patient
from fhir.resources.encounter import Encounter
from fhir.resources.identifier import Identifier
from fhir.resources.humanname import HumanName
from fhir.resources.address import Address
from fhir.resources.contactpoint import ContactPoint
from fhir.resources.period import Period
from fhir.resources.coding import Coding
from fhir.resources.reference import Reference
from fhir.resources.codeableconcept import CodeableConcept

# Configure logging
logging.basicConfig(level=logging.DEBUG, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class XMLExtractor:
    """Extracts data from HL7 XML segments"""
    
    def __init__(self, root: ET.Element):
        self.root = root
        self.ns = {'hl7': 'urn:hl7-org:v2xml'}
    
    def get_segment(self, segment_name: str) -> Optional[ET.Element]:
        """Get a segment by name (e.g., 'MSH', 'PID')"""
        return self.root.find(f".//hl7:{segment_name}", self.ns)
    
    def get_field(self, segment: ET.Element, field_path: str) -> Optional[str]:
        """Get a field value using dot notation (e.g., 'MSH.3/HD.1')"""
        try:
            # Convert path to xpath
            parts = field_path.split('/')
            xpath = "/".join(f"hl7:{p}" for p in parts)
            element = segment.find(xpath, self.ns)
            return element.text if element is not None else None
        except Exception as e:
            logger.error(f"Error extracting field {field_path}: {str(e)}")
            return None

class FHIRBuilder:
    """Builds FHIR resources from HL7 data"""
    
    @staticmethod
    def create_message_header(extractor: XMLExtractor, msh: ET.Element) -> MessageHeader:
        """Create a FHIR MessageHeader from MSH segment"""
        msg_id = extractor.get_field(msh, "MSH.10") or str(uuid.uuid4())
        source_name = extractor.get_field(msh, "MSH.3/HD.1") or "Unknown"
        msg_type = extractor.get_field(msh, "MSH.9/MSG.1") or "ADT"
        
        return MessageHeader(
            id=f"mh-{msg_id}",
            source={
                "name": source_name
            },
            eventCoding=Coding(
                system="http://terminology.hl7.org/CodeSystem/v2-0003",
                code=msg_type
            )
        )
    
    @staticmethod
    def create_patient(extractor: XMLExtractor, pid: ET.Element) -> Patient:
        """Create a FHIR Patient from PID segment"""
        patient_id = extractor.get_field(pid, "PID.3/CX.1") or str(uuid.uuid4())
        
        # Create identifier
        identifier = [Identifier(
            system="urn:oid:1.2.3.4.5.6",
            value=patient_id
        )]
        
        # Create name
        family = extractor.get_field(pid, "PID.5/XPN.1/FN.1")
        given = extractor.get_field(pid, "PID.5/XPN.2")
        name = None
        if family or given:
            name = [HumanName(
                use="official",
                family=family,
                given=[given] if given else None
            )]
        
        # Create address
        street = extractor.get_field(pid, "PID.11/XAD.1/SAD.1")
        city = extractor.get_field(pid, "PID.11/XAD.3")
        postal = extractor.get_field(pid, "PID.11/XAD.5")
        country = extractor.get_field(pid, "PID.11/XAD.6")
        
        address = None
        if any([street, city, postal, country]):
            address = [Address(
                line=[street] if street else None,
                city=city,
                postalCode=postal,
                country=country
            )]
        
        # Create patient
        patient_dict: Dict[str, Any] = {
            "id": f"pat-{patient_id}",
            "identifier": identifier
        }
        
        if name:
            patient_dict["name"] = name
        
        gender = extractor.get_field(pid, "PID.8")
        if gender == "M":
            patient_dict["gender"] = "male"
        elif gender == "F":
            patient_dict["gender"] = "female"
        
        birth_date = extractor.get_field(pid, "PID.7/TS.1")
        if birth_date and len(birth_date) >= 8:
            patient_dict["birthDate"] = datetime.strptime(birth_date[:8], '%Y%m%d').date().isoformat()
        
        if address:
            patient_dict["address"] = address
        
        phone = extractor.get_field(pid, "PID.13/XTN.1")
        if phone:
            patient_dict["telecom"] = [ContactPoint(
                system="phone",
                value=phone,
                use="home"
            )]
        
        return Patient(**patient_dict)
    
    @staticmethod
    def create_encounter(extractor: XMLExtractor, pv1: ET.Element, patient_reference: str) -> Encounter:
        """Create a FHIR Encounter from PV1 segment"""
        encounter_id = extractor.get_field(pv1, "PV1.19/CX.1") or str(uuid.uuid4())
        
        # Create encounter dictionary with required fields
        encounter_dict: Dict[str, Any] = {
            "id": f"enc-{encounter_id}",
            "status": "arrived",
            "class": {
                "system": "http://terminology.hl7.org/CodeSystem/v3-ActCode",
                "code": extractor.get_field(pv1, "PV1.2") or "AMB"
            },
            "subject": Reference(reference=patient_reference)
        }
        
        # Add period if available
        start = extractor.get_field(pv1, "PV1.44/TS.1")
        end = extractor.get_field(pv1, "PV1.45/TS.1")
        
        if start or end:
            period_dict = {}
            if start and len(start) >= 14:
                period_dict["start"] = datetime.strptime(start[:14], '%Y%m%d%H%M%S').strftime('%Y-%m-%dT%H:%M:%S+00:00')
            if end and len(end) >= 14:
                period_dict["end"] = datetime.strptime(end[:14], '%Y%m%d%H%M%S').strftime('%Y-%m-%dT%H:%M:%S+00:00')
            if period_dict:
                encounter_dict["period"] = Period(**period_dict)
        
        # Add service type if available
        service_type = extractor.get_field(pv1, "PV1.3/PL.1")
        if service_type:
            encounter_dict["serviceType"] = CodeableConcept(text=service_type)
        
        return Encounter(**encounter_dict)

class ADTMapper:
    """Maps ADT messages to FHIR resources"""
    
    @staticmethod
    def map_to_fhir(xml_string: str) -> Bundle:
        """Convert HL7 ADT XML to FHIR Bundle"""
        try:
            # Parse XML
            root = ET.fromstring(xml_string)
            extractor = XMLExtractor(root)
            
            # Create Bundle
            bundle = Bundle(
                type="message",
                entry=[]
            )
            
            # Process MSH (Message Header)
            msh = extractor.get_segment("MSH")
            if msh is not None:
                message_header = FHIRBuilder.create_message_header(extractor, msh)
                bundle.entry.append({"resource": message_header})
            
            # Process PID (Patient)
            pid = extractor.get_segment("PID")
            if pid is not None:
                patient = FHIRBuilder.create_patient(extractor, pid)
                bundle.entry.append({"resource": patient})
                
                # Process PV1 (Encounter)
                pv1 = extractor.get_segment("PV1")
                if pv1 is not None:
                    encounter = FHIRBuilder.create_encounter(extractor, pv1, f"Patient/{patient.id}")
                    bundle.entry.append({"resource": encounter})
            
            return bundle
            
        except Exception as e:
            logger.error(f"Failed to map ADT message: {str(e)}")
            logger.debug("Exception details:", exc_info=True)
            raise

def main():
    """Main entry point"""
    try:
        with open("adt_message.xml", "r") as f:
            xml_data = f.read()
        
        fhir_bundle = ADTMapper.map_to_fhir(xml_data)
        print(fhir_bundle.json(indent=2))
        
    except Exception as e:
        print(f"Error: {str(e)}")

if __name__ == "__main__":
    main()