import hl7
import requests
import logging
import time
from typing import Optional, Dict, Any
from fhir.resources.patient import Patient
from fhir.resources.identifier import Identifier
from fhir.resources.humanname import HumanName
from fhir.resources.contactpoint import ContactPoint
from fhir.resources.address import Address
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import xml.etree.ElementTree as ET
from datetime import datetime

# Configure logging with more detailed format
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - [%(filename)s:%(lineno)d] - %(message)s'
)
logger = logging.getLogger(__name__)

# Constants
FHIR_URL = "https://server.fire.ly/Patient"  # Alternative FHIR server
MIN_YEAR = 1900
MAX_YEAR = 2100
MAX_RETRIES = 3
RETRY_BACKOFF = 1

def parse_hl7_message(hl7_data: str) -> Optional[hl7.Segment]:
    """
    Parse HL7 message and extract PID segment
    
    Args:
        hl7_data (str): Raw HL7 message string
    
    Returns:
        Optional[hl7.Segment]: Parsed PID segment or None if parsing fails
    """
    try:
        message = hl7.parse(hl7_data)
        pid = message.segment("PID")
        logger.info("Successfully parsed HL7 message")
        return pid
    except Exception as e:
        logger.error(f"Failed to parse HL7 message: {str(e)}")
        return None

def safe_get_value(segment: list, index: int, subindex: Optional[tuple] = None) -> Optional[str]:
    """
    Safely extract values from HL7 segments with proper error handling
    
    Args:
        segment (list): HL7 segment
        index (int): Primary index
        subindex (tuple, optional): Tuple of sub-indices for nested access
    
    Returns:
        Optional[str]: Extracted value or None if not found
    """
    try:
        value = segment[index]
        if subindex:
            for idx in subindex:
                value = value[idx]
        return str(value) if value else None
    except (IndexError, AttributeError):
        return None

def parse_birth_date(birth_date: Optional[str]) -> Optional[str]:
    """
    Parse and validate birth date
    
    Args:
        birth_date (Optional[str]): Raw birth date string
    
    Returns:
        Optional[str]: Formatted birth date or None if invalid
    """
    if not birth_date or len(birth_date) != 8:
        return None
    
    try:
        year = int(birth_date[:4])
        month = int(birth_date[4:6])
        day = int(birth_date[6:])
        
        if (MIN_YEAR <= year <= MAX_YEAR and 
            1 <= month <= 12 and 
            1 <= day <= 31):
            return f"{year}-{month:02d}-{day:02d}"
        
        logger.warning(f"Invalid date components in birth date: {birth_date}")
        return None
    except ValueError:
        logger.warning(f"Could not parse birth date: {birth_date}")
        return None

def map_to_fhir_patient(pid: hl7.Segment) -> Optional[Patient]:
    """
    Transform HL7 PID segment to FHIR Patient
    
    Args:
        pid (hl7.Segment): PID segment from HL7 message
    
    Returns:
        Optional[Patient]: FHIR Patient resource or None if mapping fails
    """
    try:
        patient = Patient()

        # SSN (Identifier)
        ssn_value = safe_get_value(pid, 19)
        if ssn_value:
            patient.identifier = [
                Identifier(
                    use="official",
                    system="http://hl7.org/fhir/sid/us-ssn",
                    value=ssn_value
                )
            ]

        # Name
        family = safe_get_value(pid, 5, (0, 0))
        given = safe_get_value(pid, 5, (0, 1))
        if family or given:
            patient.name = [
                HumanName(
                    family=family,
                    given=[given] if given else []
                )
            ]

        # Telecom
        phone = safe_get_value(pid, 13, (0,))
        if phone:
            patient.telecom = [
                ContactPoint(
                    system="phone",
                    value=phone,
                    use="home"
                )
            ]

        # Birth Date
        birth_date = safe_get_value(pid, 7)
        patient.birthDate = parse_birth_date(birth_date)

        # Address
        address_line = safe_get_value(pid, 11, (0, 0))
        if address_line and address_line.strip():
            patient.address = [
                Address(
                    use="home",
                    line=[address_line.strip()]
                )
            ]

        # Text representation
        patient.text = {
            "status": "generated",
            "div": f"<div xmlns=\"http://www.w3.org/1999/xhtml\">Patient SSN: {ssn_value or 'N/A'}</div>"
        }

        logger.info("Successfully created FHIR Patient resource")
        return patient
    except Exception as e:
        logger.error(f"Failed to map HL7 PID segment to FHIR Patient: {str(e)}")
        return None

def create_session():
    """Create a session with retry strategy"""
    session = requests.Session()
    retry_strategy = Retry(
        total=MAX_RETRIES,
        backoff_factor=RETRY_BACKOFF,
        status_forcelist=[429, 500, 502, 503, 504]
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    return session

def send_to_fhir_server(patient: Patient) -> Optional[requests.Response]:
    """
    Send FHIR Patient to server with retry logic
    
    Args:
        patient (Patient): FHIR Patient resource
    
    Returns:
        Optional[requests.Response]: Server response or None if request fails
    """
    try:
        session = create_session()
        headers = {
            "Content-Type": "application/fhir+json",
            "Accept": "application/fhir+json",
            "Prefer": "return=representation"
        }
        
        # Convert to dict and ensure dates are strings
        patient_dict = patient.dict()
        if patient_dict.get('birthDate'):
            patient_dict['birthDate'] = patient.birthDate.isoformat() if hasattr(patient.birthDate, 'isoformat') else patient.birthDate
        
        # Remove any None values from the dictionary
        patient_dict = {k: v for k, v in patient_dict.items() if v is not None}
        
        # Log the payload for debugging
        logger.info(f"Sending payload to FHIR server: {patient_dict}")
        
        # Try to send the request with retry logic
        for attempt in range(MAX_RETRIES):
            try:
                response = session.post(
                    FHIR_URL,
                    json=patient_dict,
                    headers=headers,
                    timeout=30
                )
                
                if response.status_code >= 400:
                    logger.error(f"Server returned error: {response.status_code}")
                    logger.error(f"Response content: {response.text}")
                    # Try to parse error message for more details
                    try:
                        error_json = response.json()
                        if 'issue' in error_json:
                            for issue in error_json['issue']:
                                logger.error(f"FHIR Error: {issue.get('diagnostics', 'No details provided')}")
                    except:
                        pass
                else:
                    logger.info(f"Successfully sent FHIR Patient to server. Status: {response.status_code}")
                    return response
                
                if attempt < MAX_RETRIES - 1:
                    wait_time = (attempt + 1) * RETRY_BACKOFF
                    logger.info(f"Retrying in {wait_time} seconds...")
                    time.sleep(wait_time)
                
            except requests.exceptions.RequestException as e:
                if attempt < MAX_RETRIES - 1:
                    wait_time = (attempt + 1) * RETRY_BACKOFF
                    logger.warning(f"Request failed, retrying in {wait_time} seconds... Error: {str(e)}")
                    time.sleep(wait_time)
                else:
                    raise
        
        return response
        
    except requests.exceptions.RequestException as e:
        logger.error(f"Failed to send to FHIR server after {MAX_RETRIES} attempts: {str(e)}")
        if hasattr(e, 'response') and e.response is not None:
            logger.error(f"Full error details: {e.response.text}")
        return None
    except Exception as e:
        logger.error(f"Unexpected error: {str(e)}")
        return None
    finally:
        session.close()

def parse_hl7_xml_message(xml_string: str) -> Optional[ET.Element]:
    """Parse XML formatted HL7 message"""
    try:
        root = ET.fromstring(xml_string)
        pid = root.find(".//{urn:hl7-org:v2xml}PID")
        logger.info("Successfully parsed HL7 XML message")
        return pid
    except Exception as e:
        logger.error(f"Failed to parse HL7 XML message: {str(e)}")
        return None

def safe_get_xml_value(element: ET.Element, path: str) -> Optional[str]:
    """Safely extract values from XML elements"""
    try:
        found = element.find(f".//{path}")
        return found.text if found is not None and found.text else None
    except (AttributeError, TypeError):
        return None

def map_to_fhir_patient(pid: ET.Element) -> Optional[Patient]:
    """Transform HL7 XML PID segment to FHIR Patient"""
    try:
        patient = Patient()

        # Patient Identifier
        patient_id = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.3/CX.1")
        if patient_id:
            patient.identifier = [
                Identifier(
                    use="official",
                    system="urn:oid:1.2.3.4.5.6",  # Replace with your system
                    value=patient_id
                )
            ]

        # Name
        family = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.5/XPN.1/FN.1")
        given = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.5/XPN.2")
        if family or given:
            patient.name = [
                HumanName(
                    family=family,
                    given=[given] if given else []
                )
            ]

        # Birth Date
        birth_date = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.7/TS.1")
        if birth_date:
            try:
                # Parse HL7 date format (YYYYMMDDHHMMSS)
                parsed_date = datetime.strptime(birth_date[:8], '%Y%m%d')
                patient.birthDate = parsed_date.strftime('%Y-%m-%d')
            except ValueError:
                logger.warning(f"Could not parse birth date: {birth_date}")

        # Address
        street = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.11/XAD.1/SAD.1")
        city = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.11/XAD.3")
        postal_code = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.11/XAD.5")
        country = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.11/XAD.6")
        
        if any([street, city, postal_code, country]):
            patient.address = [
                Address(
                    use="home",
                    line=[street] if street else [],
                    city=city,
                    postalCode=postal_code,
                    country=country
                )
            ]

        # Phone
        phone = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.13/XTN.1")
        if phone:
            patient.telecom = [
                ContactPoint(
                    system="phone",
                    value=phone,
                    use="home"
                )
            ]

        # Gender
        gender = safe_get_xml_value(pid, "{urn:hl7-org:v2xml}PID.8")
        if gender:
            gender_map = {"M": "male", "F": "female"}
            patient.gender = gender_map.get(gender)

        logger.info("Successfully created FHIR Patient resource")
        return patient
    except Exception as e:
        logger.error(f"Failed to map HL7 XML PID segment to FHIR Patient: {str(e)}")
        return None

def map_adt_to_fhir_bundle(xml_string: str) -> Dict[str, Any]:
    """
    Map ADT XML message to FHIR Bundle using the ConceptMap
    
    Args:
        xml_string (str): XML string containing ADT message
    
    Returns:
        Dict[str, Any]: FHIR Bundle resource
    """
    try:
        # Parse XML
        root = ET.fromstring(xml_string)
        
        # Create Bundle
        bundle = {
            "resourceType": "Bundle",
            "type": "message",
            "entry": []
        }
        
        # Track resources for references
        resource_ids = {}
        
        # 1. Create MessageHeader from MSH
        msh = root.find(".//{urn:hl7-org:v2xml}MSH")
        if msh:
            message_header = {
                "resourceType": "MessageHeader",
                "id": f"mh-{safe_get_xml_value(msh, '{urn:hl7-org:v2xml}MSH.10')}",
                "eventCoding": {
                    "system": "http://terminology.hl7.org/CodeSystem/v2-0003",
                    "code": safe_get_xml_value(msh, "{urn:hl7-org:v2xml}MSH.9/MSG.1"),
                    "display": "ADT"
                },
                "source": {
                    "name": safe_get_xml_value(msh, "{urn:hl7-org:v2xml}MSH.3/HD.1"),
                    "endpoint": safe_get_xml_value(msh, "{urn:hl7-org:v2xml}MSH.5/HD.1")
                },
                "timestamp": safe_get_xml_value(msh, "{urn:hl7-org:v2xml}MSH.7/TS.1")
            }
            bundle["entry"].append({"resource": message_header})
            resource_ids["MessageHeader"] = message_header["id"]
            
            # Create first Provenance for message source
            provenance1 = {
                "resourceType": "Provenance",
                "id": "provenance-1",
                "recorded": message_header["timestamp"],
                "target": [{"reference": f"MessageHeader/{message_header['id']}"}],
                "agent": [{
                    "who": {
                        "identifier": {
                            "value": safe_get_xml_value(msh, "{urn:hl7-org:v2xml}MSH.3/HD.1")
                        }
                    }
                }]
            }
            bundle["entry"].append({"resource": provenance1})
        
        # 2. Create Patient from PID
        pid = root.find(".//{urn:hl7-org:v2xml}PID")
        if pid:
            patient = map_to_fhir_patient(pid)
            if patient:
                patient_dict = patient.dict()
                patient_dict["id"] = f"pat-{safe_get_xml_value(pid, '{urn:hl7-org:v2xml}PID.3/CX.1')}"
                bundle["entry"].append({"resource": patient_dict})
                resource_ids["Patient"] = patient_dict["id"]
                
                # Create Account from PID as per ConceptMap
                account = {
                    "resourceType": "Account",
                    "id": f"acc-{safe_get_xml_value(pid, '{urn:hl7-org:v2xml}PID.3/CX.1')}",
                    "subject": [{"reference": f"Patient/{patient_dict['id']}"}],
                    "status": "active"
                }
                bundle["entry"].append({"resource": account})
        
        # 3. Create Encounter from PV1
        pv1 = root.find(".//{urn:hl7-org:v2xml}PV1")
        if pv1:
            encounter = {
                "resourceType": "Encounter",
                "id": f"enc-{safe_get_xml_value(pv1, '{urn:hl7-org:v2xml}PV1.19/CX.1')}",
                "status": "arrived",  # Based on ADT A01 being admission
                "class": {
                    "system": "http://terminology.hl7.org/CodeSystem/v3-ActCode",
                    "code": safe_get_xml_value(pv1, "{urn:hl7-org:v2xml}PV1.2"),
                    "display": "Ambulatory"
                },
                "serviceType": {
                    "text": safe_get_xml_value(pv1, "{urn:hl7-org:v2xml}PV1.3/PL.1")
                },
                "period": {
                    "start": safe_get_xml_value(pv1, "{urn:hl7-org:v2xml}PV1.44/TS.1"),
                    "end": safe_get_xml_value(pv1, "{urn:hl7-org:v2xml}PV1.45/TS.1")
                }
            }
            
            # Link to Patient if exists
            if "Patient" in resource_ids:
                encounter["subject"] = {"reference": f"Patient/{resource_ids['Patient']}"}
            
            bundle["entry"].append({"resource": encounter})
            resource_ids["Encounter"] = encounter["id"]
            
            # Create EpisodeOfCare from PV1
            episode = {
                "resourceType": "EpisodeOfCare",
                "id": f"eoc-{safe_get_xml_value(pv1, '{urn:hl7-org:v2xml}PV1.19/CX.1')}",
                "status": "active",
                "patient": {"reference": f"Patient/{resource_ids['Patient']}"},
                "managingOrganization": {
                    "identifier": {
                        "value": safe_get_xml_value(pv1, "{urn:hl7-org:v2xml}PV1.3/PL.4/HD.1")
                    }
                }
            }
            bundle["entry"].append({"resource": episode})
        
        # 4. Create Provenance from EVN
        evn = root.find(".//{urn:hl7-org:v2xml}EVN")
        if evn:
            provenance3 = {
                "resourceType": "Provenance",
                "id": "provenance-3",
                "recorded": safe_get_xml_value(evn, "{urn:hl7-org:v2xml}EVN.2/TS.1"),
                "activity": {
                    "coding": [{
                        "system": "http://terminology.hl7.org/CodeSystem/v2-0003",
                        "code": safe_get_xml_value(evn, "{urn:hl7-org:v2xml}EVN.1")
                    }]
                },
                "target": []
            }
            
            # Link to created resources
            for resource_type, resource_id in resource_ids.items():
                provenance3["target"].append({"reference": f"{resource_type}/{resource_id}"})
            
            bundle["entry"].append({"resource": provenance3})
        
        # 5. Create CareTeam if PV1.ROL exists
        roles = root.findall(".//{urn:hl7-org:v2xml}ROL")
        if roles:
            care_team = {
                "resourceType": "CareTeam",
                "id": "ct-1",
                "status": "active",
                "subject": {"reference": f"Patient/{resource_ids['Patient']}"},
                "encounter": {"reference": f"Encounter/{resource_ids['Encounter']}"},
                "participant": []
            }
            
            for role in roles:
                participant = {
                    "role": [{
                        "text": safe_get_xml_value(role, "{urn:hl7-org:v2xml}ROL.3")
                    }],
                    "member": {
                        "identifier": {
                            "value": safe_get_xml_value(role, "{urn:hl7-org:v2xml}ROL.4")
                        }
                    }
                }
                care_team["participant"].append(participant)
            
            bundle["entry"].append({"resource": care_team})
        
        logger.info("Successfully created FHIR Bundle from ADT message")
        return bundle
        
    except Exception as e:
        logger.error(f"Failed to map ADT message to FHIR Bundle: {str(e)}")
        return None

if __name__ == "__main__":
    with open("server/adt_message.xml", "r") as f:
        xml_data = f.read()
    
    pid = parse_hl7_xml_message(xml_data)
    if pid:
        fhir_patient = map_to_fhir_patient(pid)
        if fhir_patient:
            response = send_to_fhir_server(fhir_patient)
            if response:
                logger.info(f"Server response: {response.status_code} - {response.text}")