import os
import logging
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
from datetime import datetime

class HL7MessageHandler(FileSystemEventHandler):
    def __init__(self, messages_dir, logs_dir):
        self.messages_dir = messages_dir
        self.logs_dir = logs_dir
        self.setup_logging()

    def setup_logging(self):
        """Setup logging configuration."""
        log_file = os.path.join(self.logs_dir, 'hl7_receiver.log')
        logging.basicConfig(
            filename=log_file,
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )

    def on_created(self, event):
        """Handle file creation events."""
        if not event.is_directory and event.src_path.endswith('.txt'):
            self.process_message(event.src_path)

    def process_message(self, filepath):
        """Process a new HL7 message file."""
        try:
            with open(filepath, 'r') as f:
                content = f.read()
            
            # Log the received message
            logging.info(f"Received HL7 message: {os.path.basename(filepath)}")
            
            # TODO: Add any additional message processing here
            
        except Exception as e:
            logging.error(f"Error processing message {filepath}: {str(e)}")

class HL7Receiver:
    def __init__(self, messages_dir=None, logs_dir=None):
        base_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
        self.messages_dir = messages_dir or os.path.join(base_dir, 'received_messages')
        self.logs_dir = logs_dir or os.path.join(base_dir, 'message_logs')
        self.observer = None
        self.handler = None

    def start(self):
        """Start the HL7 message receiver."""
        # Create directories if they don't exist
        os.makedirs(self.messages_dir, exist_ok=True)
        os.makedirs(self.logs_dir, exist_ok=True)

        # Initialize and start the observer
        self.handler = HL7MessageHandler(self.messages_dir, self.logs_dir)
        self.observer = Observer()
        self.observer.schedule(self.handler, self.messages_dir, recursive=False)
        self.observer.start()

    def stop(self):
        """Stop the HL7 message receiver."""
        if self.observer:
            self.observer.stop()
            self.observer.join() 