from datetime import datetime
import xml.etree.ElementTree as ET
from fhir.resources.patient import Patient
from fhir.resources.encounter import Encounter
from fhir.resources.humanname import HumanName
from fhir.resources.identifier import Identifier
from fhir.resources.address import Address
from fhir.resources.period import Period
from fhir.resources.reference import Reference
from fhir.resources.coding import Coding
from fhir.resources.location import Location
from fhir.resources.bundle import Bundle
from fhir.resources.organization import Organization
from fhir.resources.meta import Meta
from typing import Dict, Any
import uuid

# Define the HL7 namespace
XMLNS = {'hl7': 'urn:hl7-org:v2xml'}

def generate_uuid():
    """Generate a UUID for FHIR resources."""
    return str(uuid.uuid4())

def parse_hl7_datetime(dt_str):
    """Convert HL7 datetime string to ISO format."""
    if len(dt_str) >= 14:
        return datetime.strptime(dt_str[:14], '%Y%m%d%H%M%S').isoformat()
    return None

def extract_value(element, path):
    """Extract value from XML element using dot notation path with namespace."""
    try:
        current = element
        parts = path.split('/')
        
        # Handle the segment part (e.g., PID, PV1)
        segment = parts[0].split('.')[0]  # Get segment name (e.g., PID from PID.3)
        field_num = parts[0].split('.')[1] if '.' in parts[0] else None  # Get field number if present
        
        if len(parts) == 1:
            if field_num:
                # Find the field within the segment
                fields = current.findall(f'.//hl7:{field_num}', XMLNS)
                if fields and len(fields) > 0:
                    return fields[0].text
            else:
                # Just find the segment
                node = current.find(f'.//hl7:{segment}', XMLNS)
                return node.text if node is not None else None
        
        # Handle component parts after the segment (e.g., CX.1, XPN.1)
        if field_num:
            # First find the field
            field = current.find(f'.//hl7:{field_num}', XMLNS)
            if field is not None:
                # Then find the component
                components = parts[1].split('.')
                current_element = field
                for comp in components:
                    current_element = current_element.find(f'.//hl7:{comp}', XMLNS)
                    if current_element is None:
                        return None
                return current_element.text
        
        return None
    except (AttributeError, TypeError) as e:
        print(f"Error extracting value for path {path}: {str(e)}")
        return None

def create_organization_resource(msh_segment):
    """Create FHIR Organization resource from MSH segment."""
    organization = Organization()
    organization.id = generate_uuid()
    
    # Set meta with German base profile
    organization.meta = Meta(
        profile=["https://fhir.de/StructureDefinition/organization-de-basis"]
    )
    
    # Set identifier from MSH.5
    org_id = extract_value(msh_segment, "MSH.5/HD.1")
    if org_id:
        organization.identifier = [
            Identifier(
                system="http://hospital.org/organization-ids",
                value=org_id
            )
        ]
    
    # Set name from MSH.5
    org_name = extract_value(msh_segment, "MSH.5/HD.1")
    if org_name:
        organization.name = org_name
    
    # Set active status
    organization.active = True
    
    return organization

def create_patient_resource(pid_segment):
    """Create FHIR Patient resource from PID segment."""
    patient = Patient()
    patient.id = generate_uuid()
    
    # Set meta with German base profile
    patient.meta = Meta(
        profile=["https://fhir.de/StructureDefinition/patient-de-basis"]
    )
    
    # Set identifiers
    identifiers = []
    patient_id = extract_value(pid_segment, "PID.3/CX.1")
    if patient_id:
        identifiers.append(Identifier(
            system="http://hospital.org/patient-ids",
            value=patient_id
        ))
    patient.identifier = identifiers
    
    # Set name
    family = extract_value(pid_segment, "PID.5/XPN.1/FN.1")
    given = extract_value(pid_segment, "PID.5/XPN.2")
    if family or given:
        name = HumanName()
        if family:
            name.family = family
        if given:
            name.given = [given]
        name.use = "official"
        patient.name = [name]
    
    # Set gender
    gender = extract_value(pid_segment, "PID.8")
    if gender:
        gender_map = {"M": "male", "F": "female", "O": "other", "U": "unknown"}
        patient.gender = gender_map.get(gender, "unknown")
    
    # Set birth date
    birth_date = extract_value(pid_segment, "PID.7/TS.1")
    if birth_date and len(birth_date) >= 8:
        patient.birthDate = birth_date[:8]  # YYYYMMDD
    
    # Set address
    street = extract_value(pid_segment, "PID.11/XAD.1/SAD.1")
    city = extract_value(pid_segment, "PID.11/XAD.3")
    postal_code = extract_value(pid_segment, "PID.11/XAD.5")
    country = extract_value(pid_segment, "PID.15/CE.1")
    
    if any([street, city, postal_code, country]):
        address = Address()
        if street:
            address.line = [street]
        if city:
            address.city = city
        if postal_code:
            address.postalCode = postal_code
        if country:
            address.country = country
        address.use = "home"
        patient.address = [address]
    
    return patient

def create_encounter_resource(pv1_segment, evn_segment, patient_reference, organization_reference):
    """Create FHIR Encounter resource from PV1 and EVN segments."""
    encounter = Encounter()
    encounter.id = generate_uuid()
    
    # Set meta with German base profile
    encounter.meta = Meta(
        profile=["https://fhir.de/StructureDefinition/encounter-de-basis"]
    )
    
    # Set status (based on ADT type from EVN)
    event_type = extract_value(evn_segment, "EVN.1")
    print(f"Event type: {event_type}")  # Debug print
    
    # Default to 'in-progress' if we can't determine the status
    encounter.status = "in-progress"
    
    if event_type:
        status_map = {
            "A01": "in-progress",  # Admission
            "A03": "finished",     # Discharge
            "A04": "arrived",      # Registration
            "A08": "in-progress",  # Update
        }
        encounter.status = status_map.get(event_type, "in-progress")
    
    # Set class
    patient_class = extract_value(pv1_segment, "PV1.2")
    class_map = {
        "E": "EMER",  # Emergency
        "I": "IMP",   # Inpatient
        "O": "AMB",   # Outpatient
        "A": "AMB",   # Ambulatory
    }
    encounter.class_ = Coding(
        system="http://terminology.hl7.org/CodeSystem/v3-ActCode",
        code=class_map.get(patient_class, "AMB"),
        display="ambulatory"
    )
    
    # Set subject (patient reference)
    encounter.subject = patient_reference
    
    # Set service provider (organization reference)
    encounter.serviceProvider = organization_reference
    
    # Set period
    start_time = extract_value(pv1_segment, "PV1.44/TS.1")
    end_time = extract_value(pv1_segment, "PV1.45/TS.1")
    
    period = Period()
    if start_time:
        period.start = parse_hl7_datetime(start_time)
    if end_time:
        period.end = parse_hl7_datetime(end_time)
    encounter.period = period
    
    # Set identifier (visit number)
    visit_number = extract_value(pv1_segment, "PV1.19/CX.1")
    if visit_number:
        encounter.identifier = [
            Identifier(
                system="http://hospital.org/visit-ids",
                value=visit_number,
                type=Coding(
                    system="http://terminology.hl7.org/CodeSystem/v2-0203",
                    code="VN",
                    display="Visit Number"
                )
            )
        ]
    
    # Set location
    location_name = extract_value(pv1_segment, "PV1.3/PL.1")
    if location_name:
        location_reference = Reference(
            display=location_name
        )
        encounter.location = [{"location": location_reference}]
    
    return encounter

def create_bundle(resources: Dict[str, Any]) -> Bundle:
    """Create a FHIR Bundle containing all resources."""
    bundle = Bundle()
    bundle.id = generate_uuid()
    bundle.type = "transaction"
    
    # Set meta with German base profile
    bundle.meta = Meta(
        profile=["https://fhir.de/StructureDefinition/bundle-de-basis"]
    )
    
    entries = []
    
    # Add all resources to the bundle
    for resource_type, resource in resources.items():
        entry = {
            "resource": resource,
            "request": {
                "method": "POST",
                "url": resource_type
            }
        }
        entries.append(entry)
    
    bundle.entry = entries
    return bundle

def convert_adt_to_fhir(xml_string):
    """Convert ADT A01 message to FHIR resources."""
    # Parse XML
    root = ET.fromstring(xml_string)
    
    # Find required segments with namespace
    msh_segment = root.find(".//hl7:MSH", XMLNS)
    pid_segment = root.find(".//hl7:PID", XMLNS)
    pv1_segment = root.find(".//hl7:PV1", XMLNS)
    evn_segment = root.find(".//hl7:EVN", XMLNS)
    
    if any(segment is None for segment in [msh_segment, pid_segment, pv1_segment, evn_segment]):
        raise ValueError("Required segments MSH, PID, PV1, or EVN not found in message")
    
    # Create Organization resource
    organization = create_organization_resource(msh_segment)
    organization_reference = Reference(
        reference=f"Organization/{organization.id}"
    )
    
    # Create Patient resource
    patient = create_patient_resource(pid_segment)
    patient_reference = Reference(
        reference=f"Patient/{patient.id}"
    )
    
    # Create Encounter resource
    encounter = create_encounter_resource(pv1_segment, evn_segment, patient_reference, organization_reference)
    
    # Create Bundle with all resources
    resources = {
        "Organization": organization.dict(),
        "Patient": patient.dict(),
        "Encounter": encounter.dict()
    }
    
    bundle = create_bundle(resources)
    
    return {
        "bundle": bundle.dict(),
        "resources": resources
    }

# Example usage:
if __name__ == "__main__":
    # Read the XML file
    with open("server/adt_message.xml", "r") as f:
        xml_content = f.read()
    
    try:
        # Convert to FHIR
        fhir_output = convert_adt_to_fhir(xml_content)
        
        # Print results
        import json
        print("FHIR Bundle:")
        print(json.dumps(fhir_output["bundle"], indent=2))
        print("\nIndividual Resources:")
        for resource_type, resource in fhir_output["resources"].items():
            print(f"\n{resource_type} Resource:")
            print(json.dumps(resource, indent=2))
    except Exception as e:
        print(f"Error processing HL7 message: {str(e)}") 