import os
import re
from lxml import etree
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

class XMLStructureParser:
    def __init__(self):
        self._resources = {}

    def parse_xml_file(self, filepath):
        """Parse an XML file and extract its structure."""
        try:
            with open(filepath, 'r') as f:
                content = f.read()

            # Extract comments using regex
            comments = {}
            comment_pattern = r'<!--(.*?)-->'
            for match in re.finditer(comment_pattern, content):
                comment = match.group(1).strip()
                line_number = content[:match.start()].count('\n') + 1
                comments[line_number] = comment

            # Parse XML
            parser = etree.XMLParser(remove_blank_text=True)
            tree = etree.parse(filepath, parser)
            root = tree.getroot()

            # Process the XML structure
            structure = self._process_element(root, comments)

            # Store the resource
            resource_key = os.path.splitext(os.path.basename(filepath))[0]
            self._resources[resource_key] = {
                'filename': os.path.basename(filepath),
                'path': filepath,
                'structure': structure
            }

            return self._resources[resource_key]

        except Exception as e:
            print(f"Error parsing XML file {filepath}: {str(e)}")
            return None

    def _process_element(self, element, comments):
        """Process an XML element and its children recursively."""
        result = {
            'tag': element.tag,
            'attributes': dict(element.attrib),
            'comments': [],
            'children': []
        }

        # Get element line number
        line_number = element.sourceline if hasattr(element, 'sourceline') else None

        # Add comments associated with this element
        if line_number in comments:
            result['comments'].append(comments[line_number])

        # Process text content
        if element.text and element.text.strip():
            result['text'] = element.text.strip()

        # Process children
        for child in element:
            child_result = self._process_element(child, comments)
            if child_result:
                result['children'].append(child_result)

        return result

class XMLFileWatcher(FileSystemEventHandler):
    def __init__(self, xml_dir, processor):
        self.xml_dir = xml_dir
        self.processor = processor

    def on_created(self, event):
        """Handle file creation events."""
        if not event.is_directory and event.src_path.endswith('.xml'):
            self.processor.parse_xml_file(event.src_path)

    def on_modified(self, event):
        """Handle file modification events."""
        if not event.is_directory and event.src_path.endswith('.xml'):
            self.processor.parse_xml_file(event.src_path) 